<?php
/**
 * @file
 * Contains information about hooks provided by Selective Tweets.
 */

/**
 * Several Selective Tweets block settings such as Tweet filters can be altered.
 * Changes to $data are passed by reference. No return value is needed.
 * Potential tokens have already been replaced in this phase.
 *
 * @param $data
 *   Array containing alterable settings.
 *     - screen_name
 *     - from_account
 *     - to_account
 *     - user_mentions
 *     - hashtags
 *     - load_more_text
 *     - loading_text
 *     - empty_text
 * @param $context
 *   Array containing contextual information:
 *     - block_delta: The unique machine name of the Selective Tweets block.
 *     - context_entity: Object used for token replacements. Can be NULL.
 */
function hook_selective_tweets_block_alter(&$data, $context) {
  // Example: always include hashtag 'drupal' in de hashtag filter of the
  // Selective Tweets block with delta 'my_selective_tweets_block'.
  if ($context['block_delta'] == 'my_selective_tweets_block') {
    $data['hashtags'] .= empty($data['hashtags']) ? 'drupal' : ' AND drupal';
  }

  // Example: Change empty texts of all Selective Tweets block on the website,
  // despite their individual settings.
  $data['empty_text'] = t('Go home, there are no tweets here.');
}
